﻿using MindFusion.Charting;
using MindFusion.Drawing;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Xamarin.Forms;

namespace BarChart3D
{
	// Learn more about making custom code visible in the Xamarin.Forms previewer
	// by visiting https://aka.ms/xamarinforms-previewer
	[DesignTimeVisible(false)]
	public partial class MainPage : ContentPage
	{
		public MainPage()
		{
			InitializeComponent();
            barChart.Title = "Agricultural produce by type";

            // create sample data
            barChart.Series = GetSeriesCollection();

			barChart.Theme.CommonSeriesFills = barChart.Theme.CommonSeriesStrokes = new List<Brush> {
				new SolidBrush(Color.FromRgb(0, 52, 102)),
				new SolidBrush(Color.FromRgb(156, 170, 198)),
				new SolidBrush(Color.FromRgb(206, 0, 0))
			};

			// background
			barChart.BackgroundColor = Colors.White;
			barChart.Theme.PlotBackground = new SolidBrush(Color.FromRgb(245, 245, 245));          
			barChart.Theme.DataLabelsBrush = new SolidBrush(Color.FromRgb(100, 100, 100));
			barChart.Theme.AxisLabelsBrush = new SolidBrush(Color.FromRgb(100, 100, 100));
			barChart.Theme.AxisTitleBrush = new SolidBrush(Color.FromRgb(100, 100, 100));
			barChart.Theme.DataLabelsFontSize = 10;
			barChart.Theme.AxisLabelsFontSize = 10;
			barChart.Theme.AxisTitleFontSize = 14;
			barChart.Theme.LegendBackground = new SolidBrush(Color.FromRgb(230, 230, 230));
			barChart.Theme.LegendBorderStroke = new SolidBrush(Color.FromRgb(100, 100, 100));
			barChart.Theme.LegendTitleBrush = new SolidBrush(Color.FromRgb(100, 100, 100));
			barChart.Theme.LegendBorderStrokeThickness = 1;
			barChart.Theme.AxisStroke = new SolidBrush(Color.FromRgb(100, 100, 100));

			// axis appearance
			barChart.XAxis.Title = "";
            barChart.YAxis.Title = "Quantity in tons";
            barChart.Theme.AxisStrokeThickness = 1;

            barChart.XAxis.MinValue = 25;
            barChart.XAxis.MaxValue = 45;

            // legend appearance
            barChart.LegendTitle = "Agricultural type";
            barChart.LegendMargin = new Margins(80, 5, 5, 5);

            barChart.LayoutPanel.Margin = new Margins(10);

            SetupControls();
        }

        void SetupControls()
        {
            cbxBarLayout.ItemsSource = new[] {
                BarLayout.Overlay,
                BarLayout.SideBySide,
                BarLayout.Stack };
            cbxBarLayout.SelectedItem = barChart.BarLayout;
        }

        ObservableCollection<Series> GetSeriesCollection()
        {
            var labels = new[] { "Tomatoes", "Cucumbers", "Peppers", "Lettuce" };
            var collection = new ObservableCollection<Series>();
            for (int i = 0; i < 3; i++)
            {
                if (i == 0)
                {
                    collection.Add(new Series2D(new double[] { 20, 30, 40, 50 },
                        new double[] { 50, 40, 50, 15 },
                        labels)
                    { Title = "Traditional" });
                }
                if (i == 1)
                {
                    collection.Add(new Series2D(new double[] { 20, 30, 40, 50 },
                       new double[] { 60, 15, 20, 80 },
                       labels)
                    { Title = "Urban" });
                }
                if (i == 2)
                {
                    collection.Add(new Series2D(new double[] { 20, 30, 40, 50 },
                       new double[] { 15, 60, 15, 90 },
                       labels)
                    { Title = "Hydroponics" });
                }
            }
            return collection;
        }

        void cbxBarLayout_ItemSelected(object sender, SelectedItemChangedEventArgs e)
        {
            barChart.BarLayout = (BarLayout)cbxBarLayout.SelectedItem;
        }
    }
}
