﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Xamarin.Forms;

using MindFusion.Diagramming;
using MindFusion.Drawing;


namespace AnchorPoints
{
	// Learn more about making custom code visible in the Xamarin.Forms previewer
	// by visiting https://aka.ms/xamarinforms-previewer
	[DesignTimeVisible(false)]
	public partial class MainPage : ContentPage
	{
		public MainPage()
		{
			InitializeComponent();
		}

        protected override void OnAppearing()
        {
            base.OnAppearing();

			diagram = diagramView.Diagram;

			diagram.AlignToGrid = false;
			diagram.AllowSelfLoops = false;
			diagram.BackBrush = new SolidBrush(Color.FromHex("#FFE0E0E0"));
			diagram.DiagramStyle.ShadowBrush = new SolidBrush(Color.FromHex("#96BEBEBE"));
			diagram.LinkSegments = ((short)(3));
			diagram.LinkShape = LinkShape.Cascading;
			diagram.RouteLinks = true;
			diagram.SelectAfterCreate = false;
			diagram.ShapeHandlesStyle = HandlesStyle.DashFrame;
			diagram.SnapToAnchor = SnapToAnchor.OnCreateOrModify;
			diagram.NodeEffects.Add(new GlassEffect() { Type = GlassEffectType.Type1 });

			diagram.ShapeNodeStyle.Brush = new LinearGradientBrush(
				Color.FromRgb(102, 154, 204), Color.FromRgb(0, 52, 102), 30);
			diagram.DiagramLinkStyle.Stroke = new MindFusion.Drawing.SolidBrush(Color.FromRgb(100, 100, 100));
			diagram.DiagramLinkStyle.HeadStroke = new MindFusion.Drawing.SolidBrush(Color.FromRgb(100, 100, 100));
			diagram.BackBrush = new MindFusion.Drawing.SolidBrush(Color.FromRgb(224, 233, 233));
			ShapeNode pb1, pb2, pb3, pb4, decb1, decb2;

			apat1 = new AnchorPattern(new AnchorPoint[]
				{
					new AnchorPoint(50, 0, true, true),
					new AnchorPoint(100, 50, true, true),
					new AnchorPoint(50, 100, true, true),
					new AnchorPoint(0, 50, true, true)
				});

			apat2 = new AnchorPattern(new AnchorPoint[]
				{
					new AnchorPoint(10, 0, true, false, MarkStyle.Circle, Color.RoyalBlue),
					new AnchorPoint(50, 0, true, false, MarkStyle.Circle, Color.Blue),
					new AnchorPoint(90, 0, true, false, MarkStyle.Circle, Color.Firebrick),
					new AnchorPoint(10, 100, false, true, MarkStyle.Rectangle),
					new AnchorPoint(50, 100, false, true, MarkStyle.Rectangle),
					new AnchorPoint(90, 100, false, true, MarkStyle.Rectangle),
					new AnchorPoint(0, 50, true, true, MarkStyle.Custom)
				});

			pb1 = new ShapeNode(diagram);
			pb1.Bounds = new Rectangle(10, 7, 25, 18);
			pb1.Shape = Shapes.Ellipse;
			pb1.Text = "Start";
			pb1.AnchorPattern = apat1;
			diagram.Nodes.Add(pb1);

			pb2 = new ShapeNode(diagram);
			pb2.Bounds = new Rectangle(20, 75, 25, 18);
			pb2.Text = "node 1";
			pb2.AnchorPattern = apat2;
			diagram.Nodes.Add(pb2);

			pb3 = new ShapeNode(diagram);
			pb3.Bounds = new Rectangle(70, 70, 25, 18);
			pb3.Text = "node 2";
			pb3.AnchorPattern = apat2;
			diagram.Nodes.Add(pb3);

			pb4 = new ShapeNode(diagram);
			pb4.Bounds = new Rectangle(80, 100, 25, 18);
			pb4.Shape = Shapes.Ellipse;
			pb4.Text = "End";
			pb4.AnchorPattern = apat1;
			diagram.Nodes.Add(pb4);

			decb1 = new ShapeNode(diagram);
			decb1.Bounds = new Rectangle(20, 35, 30, 20);
			decb1.Shape = Shapes.Decision;
			decb1.Text = "check 1";
			decb1.AnchorPattern = AnchorPattern.Decision1In3Out;
			diagram.Nodes.Add(decb1);

			decb2 = new ShapeNode(diagram);
			decb2.Bounds = new Rectangle(70, 30, 30, 20);
			decb2.Shape = Shapes.Decision;
			decb2.Text = "check 2";
			decb2.AnchorPattern = AnchorPattern.Decision2In2Out;
			diagram.Nodes.Add(decb2);

			diagram.Links.Add(new DiagramLink(diagram, decb1, decb2));

			if (MindFusion.PlatformProvider.PlatformType == MindFusion.PlatformType.Android)
				diagramView.SizeChanged += (s, e) => { ZoomToFit(); };
			else if (MindFusion.PlatformProvider.PlatformType == MindFusion.PlatformType.IOS)
				ZoomToFit();

		}

		private void ZoomToFit()
		{
			if (diagramView.Bounds.IsEmpty || diagramView.Bounds.Width < 0 || diagramView.Bounds.Height < 0)
				return;
			var viewport = MindFusion.Utilities.DeviceToDoc(
				diagramView.Diagram.CreateMeasureGraphics(), diagramView.Bounds);
			if (viewport.Width < diagramView.Diagram.GetContentBounds(false, false).Width)
				diagramView.ZoomToFit();
		}

		AnchorPattern apat1;
		AnchorPattern apat2;
		Diagram diagram;
	}
}
